package net.wanji.utc.common.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.mapper.ApiInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.ApiInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
@Slf4j
@EnableAsync
public class ApiInfoTask {
    @Autowired
    ApiInfoMapper apiInfoMapper;
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;

    @Scheduled(fixedRate = 2 * 60 * 1000)
    public void apiInfoRefresh() {
        try {
            ApiInfoPO apiInfoPO = new ApiInfoPO();
            List<ApiInfoPO> apiInfoPOList = apiInfoMapper.selectAll();
            if (apiInfoPOList== null || apiInfoPOList.isEmpty()) {
                return;
            }
            // 刷新JVM
            for (ApiInfoPO infoPO : apiInfoPOList) {
                Integer manufacturerId = infoPO.getManufacturerId();
                ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
                String abbr = manufacturerInfoPO.getNickName();
                String s = abbr + Constants.SEPARATOR_UNDER_LINE + infoPO.getCode();
                Constants.putManufacturerUrlMap(s, infoPO);
            }
            log.info("刷新" + apiInfoPOList.size() + "条厂商URL至JVM");
        } catch (Exception e) {
            log.error("刷新厂商URL到JVM异常", e);
        }
    }
}