package net.wanji.utc.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.common.typeenum.HkLightsDirEnum;
import net.wanji.utc.common.typeenum.HkLightsTurnEnum;
import net.wanji.utc.common.typeenum.HkLightsTypeEnum;
import net.wanji.utc.entity.cache.PhaseCache;
import net.wanji.utc.mapper.CrossLightsMapper;
import net.wanji.utc.mapper.CrossPhaseLightsMapper;
import net.wanji.utc.mapper.CrossPhaseMapper;
import net.wanji.utc.mapper.CrossSchemeMapper;
import net.wanji.utc.po.CrossLightsPO;
import net.wanji.utc.po.CrossPhaseLightsPO;
import net.wanji.utc.po.CrossPhasePO;
import net.wanji.utc.po.CrossSchemePO;
import net.wanji.utc.service.HkSchemePhaseLightsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 方案数据-方案信息、相位信息
 *
 * @author Kent HAN
 * @date 2022/11/16 15:27
 */
@Service
public class HkSchemePhaseLightsServiceImpl implements HkSchemePhaseLightsService {
    // 相位信息缓存，key-相位号，value-相位信息对象
    public static Map<Integer, PhaseCache> phaseCacheMap = new HashMap<>();

    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;
    @Autowired
    private ArtemisConfig artemisConfig;
    @Autowired
    CrossSchemeMapper crossSchemeMapper;
    @Autowired
    CrossPhaseMapper crossPhaseMapper;
    @Autowired
    CrossLightsMapper crossLightsMapper;
    @Autowired
    CrossPhaseLightsMapper crossPhaseLightsMapper;

    @Override
    public void hkSchemePhaseLights(String crossId, String crossCode) throws Exception {
        List<CrossSchemePO> crossSchemePOList = new ArrayList<>();

        // 请求3.1.3接口，获取并处理方案数据
        Map<String, String> path = getPathMapByApiCode("uploadPatternCodeList");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        String Result = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject object = JSON.parseObject(Result);
        if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
            JSONArray data = object.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                JSONObject obj =  data.getJSONObject(i);
                Integer patternNo = obj.getInteger("patternNo");
                String patternName = obj.getString("patternName");
                // 请求3.1.4接口，获取并处理方案数据
                handleSchemeData(crossId, crossCode, crossSchemePOList, patternNo, patternName);
            }
        }
        // 请求3.1.11接口，获取并处理相位数据
        handlePhaseData(crossId, crossCode);
        // 请求3.1.2接口，获取并处理灯组数据
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = handleLightsData(crossId, crossCode);
        // 更新相位-灯组关系
        crossPhaseLightsMapper.deleteByCrossId(crossId);
        crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
    }

    private List<CrossPhaseLightsPO> handleLightsData(String crossId, String crossCode) throws Exception {
        Map<String, String> path = getPathMapByApiCode("getCrossInfo");
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = new ArrayList<>();
        JSONObject requestContent = new JSONObject();
        // 海康固定入参字段名
        requestContent.put("pageNo", 1);
        requestContent.put("pageSize", 9999);
        JSONObject searchObj = new JSONObject();
        requestContent.put("searchObj", searchObj);
        ArrayList<String> codeLst = new ArrayList<>();
        codeLst.add(crossCode);
        searchObj.put("codeLst", codeLst);
        String body = requestContent.toJSONString();
        // 请求参数
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null,
                null, "application/json", null);
        JSONObject responseObj = JSON.parseObject(responseStr);
        if(Constants.HK_SUCCESS_CODE.equals(responseObj.getInteger(Constants.HK_CODE_KEY))) {
            JSONObject data = responseObj.getJSONObject("data");
            JSONArray list = data.getJSONArray("list");
            for(int i = 0; i < list.size(); i++){
                JSONObject crossObj = list.getJSONObject(i);
                JSONArray laneEles = crossObj.getJSONArray("laneEles");
                for (int j = 0; j < laneEles.size(); j++) {
                    JSONObject laneObj = laneEles.getJSONObject(j);
                    CrossLightsPO crossLightsPO = new CrossLightsPO();
                    String lightsNo = laneObj.getString("elementId");
                    crossLightsPO.setLightsNo(lightsNo);
                    Integer lampGroupType = laneObj.getInteger("lampGroupType");
                    Integer type = HkLightsTypeEnum.getCodeByHkCode(lampGroupType);
                    crossLightsPO.setType(type);
                    Integer direction = laneObj.getInteger("direction");
                    Integer dir = HkLightsDirEnum.getCodeByHkCode(direction);
                    crossLightsPO.setDir(dir);
                    Integer turn = laneObj.getInteger("turn");
                    turn = HkLightsTurnEnum.getCodeByHkCode(turn);
                    crossLightsPO.setTurn(turn);
                    crossLightsPO.setCrossId(crossId);
                    crossLightsPO.setInOutType(3); // 海康都是进出口
                    // 插入数据返回自增ID
                    crossLightsMapper.deleteOne(crossId, lightsNo);
                    crossLightsMapper.insertOne(crossLightsPO);
                    Integer crossLightsPOId = crossLightsPO.getId();
                    // 匹配缓存信息，构造相位灯组关系对象列表
                    Integer laneNo = laneObj.getInteger("laneNo");
                    Collection<PhaseCache> phaseCaches = phaseCacheMap.values();
                    for (PhaseCache phaseCache : phaseCaches) {
                        List<Integer> laneNoList = phaseCache.getLaneNoList();
                        for (Integer laneNoInCache : laneNoList) {
                            if (Objects.equals(laneNo, laneNoInCache)) {
                                CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                                crossPhaseLightsPO.setLightsId(crossLightsPOId);
                                crossPhaseLightsPO.setPhaseId(phaseCache.getPhaseId());
                                crossPhaseLightsPO.setCrossId(crossId);
                                crossPhaseLightsPOList.add(crossPhaseLightsPO);
                            }
                        }
                    }
                }
            }
        }
        return crossPhaseLightsPOList;
    }

    private void handlePhaseData(String crossId, String crossCode) throws Exception {
        Map<String, String> path = getPathMapByApiCode("getPhaseRlatLanes");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        String Result = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject object = JSON.parseObject(Result);
        if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
            JSONObject data = object.getJSONObject("data");
            JSONArray list = data.getJSONArray("phaseList");
            for (int k = 0; k < list.size(); k++) {
                CrossPhasePO crossPhasePO = new CrossPhasePO();
                JSONObject phase = list.getJSONObject(k);
                Integer phaseNo = phase.getInteger("phaseNo");
                // 从缓存中获取数据
                PhaseCache phaseCache = phaseCacheMap.get(phaseNo);
                crossPhasePO.setPhaseNo(phaseCache.getPhaseNo().toString());
                crossPhasePO.setName(phaseCache.getPhaseName());
                crossPhasePO.setSort(phaseCache.getPhaseSequence());
                crossPhasePO.setCrossId(crossId);
                crossPhasePO.setPlanId(phaseCache.getSchemeId());
                crossPhasePO.setRingNo(phaseCache.getRingNo());
                crossPhasePO.setControlMode(1); // 海康均为定周期
                Integer phaseLength = phaseCache.getPhaseLength();
                crossPhasePO.setPhaseTime(phaseLength);
                // 从接口返回对象中获取数据
                Integer greenFlash = phase.getInteger("greenFlash");
                crossPhasePO.setGreenFlashTime(greenFlash);
                Integer yellow = phase.getInteger("yellow");
                crossPhasePO.setYellowTime(yellow);
                Integer allred = phase.getInteger("allred");
                crossPhasePO.setRedTime(allred);
                int green = phaseLength - greenFlash - yellow - allred;
                crossPhasePO.setGreenTime(green);
                Integer minGreen = phase.getInteger("minGreen");
                crossPhasePO.setMinGreenTime(minGreen);
                Integer maxGreen = phase.getInteger("maxGreen");
                crossPhasePO.setMaxGreenTime(maxGreen);
                // 插入数据返回自增ID，更新缓存
                crossPhaseMapper.deleteOne(crossId, phaseNo.toString());
                crossPhaseMapper.insertOne(crossPhasePO);
                Integer crossPhasePOId = crossPhasePO.getId();
                phaseCache.setPhaseId(crossPhasePOId);
            }
        }
    }

    private void handleSchemeData(String crossId, String crossCode, List<CrossSchemePO> crossSchemePOList, Integer patternNo, String patternName) throws Exception {
        Map<String, String> path = getPathMapByApiCode("getCrossPlanDetail");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        queryMap.put("patternNo", patternNo.toString());
        String Result = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject object = JSON.parseObject(Result);
        if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
            JSONArray data = object.getJSONArray("data");
            for (int j = 0; j < data.size(); j++) {
                JSONObject obj =  data.getJSONObject(j);
                Integer cycle = obj.getInteger("cycle");
                Integer coordPhase = obj.getInteger("coordPhase");
                Integer offset = obj.getInteger("offset");
                // 构造数据库记录
                CrossSchemePO crossSchemePO = new CrossSchemePO();
                crossSchemePO.setSchemeNo(patternNo.toString());
                crossSchemePO.setName(patternName);
                crossSchemePO.setCrossId(crossId);
                crossSchemePO.setCycle(cycle);
                crossSchemePO.setCoordPhase(coordPhase);
                crossSchemePO.setOffset(offset);
                crossSchemePO.setSource(2);
                crossSchemePO.setIsDeleted(0);
                crossSchemePOList.add(crossSchemePO);
                // 插入数据返回自增ID
                crossSchemeMapper.deleteOne(crossId, patternNo.toString());
                crossSchemeMapper.insertOne(crossSchemePO);
                Integer schemePOId = crossSchemePO.getId();
                // 缓存相位信息
                putPhaseCache(obj, schemePOId);
            }
        }
    }

    private void putPhaseCache(JSONObject obj, Integer schemePOId) {
        phaseCacheMap.clear();
        JSONArray phases = obj.getJSONArray("phaseInfos");
        for (int i = 0; i < phases.size(); i++) {
            JSONObject phase = phases.getJSONObject(i);
            PhaseCache phaseCache = new PhaseCache();
            Integer ringNo = phase.getInteger("ringNo");
            phaseCache.setRingNo(ringNo);
            Integer phaseNo = phase.getInteger("phaseNo");
            phaseCache.setPhaseNo(phaseNo);
            String phaseName = phase.getString("phaseName");
            phaseCache.setPhaseName(phaseName);
            Integer phaseSequence = phase.getInteger("phaseSequence");
            phaseCache.setPhaseSequence(phaseSequence);
            Integer phaseLength = phase.getInteger("phaseLength");
            phaseCache.setPhaseLength(phaseLength);
            phaseCache.setSchemeId(schemePOId);
            // 获取车道列表
            JSONArray lanes = phase.getJSONArray("lanes");
            List<Integer> laneNoList = new ArrayList<>();
            for (int j = 0; j < lanes.size(); j++) {
                JSONObject lane = lanes.getJSONObject(j);
                Integer laneNo = lane.getInteger("laneNo");
                laneNoList.add(laneNo);
            }
            phaseCache.setLaneNoList(laneNoList);
            phaseCacheMap.put(phaseNo, phaseCache);
        }
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getAbbr(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }
}
