package net.wanji.utc.service.impl;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.service.HkStaticInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @date 2022/11/15 13:18
 */
@Slf4j
@Service
public class HkStaticInfoServiceImpl implements HkStaticInfoService {
    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;
    @Autowired
    private ArtemisConfig artemisConfig;
    @Autowired
    private CrossInfoMapper crossInfoMapper;
    @Autowired
    private ManufacturerInfoMapper manufacturerInfoMapper;

    @Override
    public List<CrossInfoPO> hkCrossBasicInfo() {
        List<CrossInfoPO> res = new ArrayList<>();
        try{
            // key如"http://"，value如"/artemis/api/itscms-scms/v1/crossinfo/hkCrossBasicInfo"
            Map<String, String> path = getPathMapByApiCode("crossBasicInfo");
            JSONObject requestContent = new JSONObject();
            // 海康固定入参字段名
            requestContent.put("pageNo", 1);
            requestContent.put("pageSize", 9999);
            requestContent.put("searchObj", new JSONObject());
            String body = requestContent.toJSONString();
            // 请求参数
            String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null,
                    null, "application/json", null);
            JSONObject responseObj = JSON.parseObject(responseStr);

            if(Constants.HK_SUCCESS_CODE.equals(responseObj.getInteger(Constants.HK_CODE_KEY))) {
                JSONObject data = responseObj.getJSONObject("data");
                JSONArray list = data.getJSONArray("list");
                for(int i = 0; i < list.size(); i++){
                    JSONObject signalObj = list.getJSONObject(i);
                    String crossCode = signalObj.getString("crossCode");
                    String crossName = signalObj.getString("crossName");
                    // t_cross_info表的主键ID 例如"18c8db5a7772428eb91be749fb3004c8"
                    CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossCode);
                    // 如果数据库中已经有记录，则跳过
                    if (null != crossInfoPO) {
                        continue;
                    }
                    crossInfoPO = new CrossInfoPO();
                    // 32位随机ID
                    crossInfoPO.setId(IdUtil.simpleUUID());
                    crossInfoPO.setName(crossName);
                    crossInfoPO.setCode(crossCode);
                    ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByAbbr(
                            BasicEnum.ManufacturerEnum.HK.getAbbr());
                    if (manufacturerInfoPO != null) {
                        crossInfoPO.setManufacturerId(manufacturerInfoPO.getId());
                    }
                    crossInfoPO.setIp(signalObj.getString("ip"));
                    crossInfoPO.setPort(signalObj.getInteger("port"));
                    // 经纬度
                    String longitude = signalObj.getString("longitude");
                    String latitude = signalObj.getString("latitude");
                    crossInfoPO.setLocation(longitude + "," + latitude);
                    crossInfoPO.setVersion(signalObj.getString("sigVersion"));
                    crossInfoPO.setInstallTime(signalObj.getDate("installTime"));

                    res.add(crossInfoPO);
                }
            }
        }  catch(Exception e) {
            log.error("服务器端内部错误！", e);
        }
        return res;
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getAbbr(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }

}
