package net.wanji.utc.service.impl;

import net.wanji.utc.common.genericentity.ManufacturerRes;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.service.HkStaticInfoService;
import net.wanji.utc.service.OthersStaticInfoService;
import net.wanji.utc.service.HkSchemePhaseLightsService;
import net.wanji.utc.service.StaticInfoService;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.genericentity.OutVO;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.vo.DetailCrossInfoVO;
import net.wanji.utc.vo.SchemePhaseLightsInVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/15 10:32
 */
@Service
public class StaticInfoServiceImpl implements StaticInfoService {
    @Autowired
    HkStaticInfoService hkStaticInfoService;
    @Autowired
    CrossInfoMapper crossInfoMapper;
    @Autowired
    OthersStaticInfoService othersStaticInfoService;
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;
    @Autowired
    HkSchemePhaseLightsService hkSchemePhaseLightsService;

    @Override
    public ManufacturerRes<DetailCrossInfoVO> crossBasicInfo(OutVO<BaseCrossInfo> outVO) {
        List<CrossInfoPO> crossInfoPOList = new ArrayList<>();
        ManufacturerRes<DetailCrossInfoVO> res = new ManufacturerRes<>();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getAbbr(), outVO.getManufacturerAbbr())) {
            crossInfoPOList = hkStaticInfoService.hkCrossBasicInfo();
        } else {
            // todo 其他厂商
            res = othersStaticInfoService.crossBasicInfo(outVO.getManufacturerAbbr());
            if (null == res) {
                return null;
            }
            List<DetailCrossInfoVO> dataContent = res.getDatalist();
            if (ListUtil.isEmpty(dataContent)) {
                return null;
            }
            for (DetailCrossInfoVO vo : dataContent) {
                String id = vo.getTelesemeId();
                CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(id);
                if (null != crossInfoPO) {
                    continue;
                }
                crossInfoPO = new CrossInfoPO();
                // 用信号机ID作为路口ID
                crossInfoPO.setId(id);
                crossInfoPO.setName(vo.getCrossName());
                crossInfoPO.setCode(id);
                ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByAbbr(vo.getManufacturerAbbr());
                if (manufacturerInfoPO != null) {
                    crossInfoPO.setManufacturerId(manufacturerInfoPO.getId());
                }
                crossInfoPO.setIp(vo.getIp());
                crossInfoPO.setPort(vo.getPort());
                // 经纬度
                Double longitude = vo.getLon();
                Double latitude = vo.getLat();
                crossInfoPO.setLocation(longitude + "," + latitude);
                crossInfoPO.setVersion(vo.getVersion());
                crossInfoPO.setModel(vo.getModel());
                crossInfoPO.setInstallTime(vo.getInstallTime());

                crossInfoPOList.add(crossInfoPO);
            }
        }
        //存储信号机信息
        if (crossInfoPOList.size() == 0) {
            return null;
        }
        crossInfoMapper.insertBatch(crossInfoPOList);
        return res;
    }

    @Override
    public void schemePhaseLights(SchemePhaseLightsInVO schemePhaseLightsInVO) throws Exception {
        String crossId = schemePhaseLightsInVO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商缩写
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerAbbr = manufacturerInfoPO.getNickName();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getAbbr(), manufacturerAbbr)) {
            // 海康
            hkSchemePhaseLightsService.hkSchemePhaseLights(crossId, crossCode);
        } else {
            // todo 其他厂商

        }
    }
}
